#ifndef __EFFECTENGINE_H__
#define __EFFECTENGINE_H__

#include <e32base.h>

#include "DolbyHeadphone.h"

class MMyDolbyHeadphoneObserver2
{
public:
  virtual void DolbyChange2(TBool aState)=0;
};

enum TFoldPlayerType
{
	EFFPtNone,
	EFFPtInternal,
	EFFPtExternal
};

const TUid KCRUidMPXMPSettings = {0x101FFCDC};
const TUint32 KMPXMPEqPresetId = 0x00000001;

 //forward declaration
class CRepository;
class CAudioEqualizerUtility;
class CMdaAudioPlayerUtility;
class CMdaAudioOutputStream;

//a observer interface for Preset change
class MEqPresetWatcher
{
       public:
               virtual void EqPresetChanged(TInt aPreset) = 0;
};


class CEqPresetNotifyHandler : public CActive
{
 public:
       static CEqPresetNotifyHandler* NewL( MEqPresetWatcher& aObserver);

       void StartObservingL();
       void StopObserving();

       virtual ~CEqPresetNotifyHandler();

 private:
       CEqPresetNotifyHandler(MEqPresetWatcher& aObserver);
       void ConstructL();

       void RunL();
       void DoCancel();

 private:
       CRepository* iRepository;
       TUint32 iKey;
       TUid iUid;
       MEqPresetWatcher& iObserver; //class using the observer
};

//////////////////////////////////////////////////////////////

class CAudioEffectEngine : public CBase,
				public MMyDolbyHeadphoneObserver
{
public: 

    static CAudioEffectEngine* NewL(CMdaAudioPlayerUtility* aPlayer, TBool aDolbySupported, MMyDolbyHeadphoneObserver2& aObs);
    static CAudioEffectEngine* NewL(CMdaAudioOutputStream* aAudioOutput, TBool aDolbySupported, MMyDolbyHeadphoneObserver2& aObs);
	
    ~CAudioEffectEngine();
	
    void CreateAudioEffectsL();

    void DestroyAudioEffect();

    void SetEqualizerL(TInt aPreset);
   
    void SetDolbyHeadphoneL(TBool aOn);

private:
    CAudioEffectEngine(TBool aDolbySupported, MMyDolbyHeadphoneObserver2& aObs);

    void DolbyConstructL();

    void ConstructL(CMdaAudioPlayerUtility* aPlayer);
    void ConstructL(CMdaAudioOutputStream* aAudioOutput);

    void DolbyChange(TBool aState);

private: //Data
    CMdaAudioPlayerUtility* iMdaPlayer;  // Not owned
    CMdaAudioOutputStream*  iAudioOutput;// Not owned
	
    CAudioEqualizerUtility* iEqualizerEffect;
    TFoldPlayerType         iType;

    CMyDolbyHeadphone*      iDolbyHeadphone;
    TBool iDolbySupported;
    MMyDolbyHeadphoneObserver2& iObs;
};

#endif // __EFFECTENGINE_H__
