#ifndef __FOLDERPLAYVIEW_h__
#define __FOLDERPLAYVIEW_h__

// INCLUDES
#include <coecntrl.h>
#include <aknlists.h>
#include <charconv.h>
#include <aknbutton.h>

#include <remconcoreapitargetobserver.h>
#include <remconinterfaceselector.h>

#include <hwrmlight.h>

#include "autoresumehandler.h"

#include "EffectEngine.h"

#include <accmonitor.h>

#include "DiskDetector.h"

#include "musicprogressbar.h"

// FORWARD DECLARATIONS
class CAknVolumePopup;
class CAknTitlePane;

class CFolderPlayAppUi;
class CPlayEngine;
class CSimpleAudioPlayer;
class CRemConKeyResponse;

enum TFastForwardRewind
{
	EFFRwNone,
	EFFType,
	ERwType
};

/*
enum TFoldPlayerType
{
	EFFPtNone,
	EFFPtInternal,
	EFFPtExternal
};
*/

class CFolderView : public CCoeControl,
			public MEikListBoxObserver,
                        public MCoeControlObserver,
			public MRemConCoreApiTargetObserver,
			public MEqPresetWatcher,
                        public MHWRMLightObserver,
			public MAccMonitorObserver,
			public CDiskDetector::MObserver 
{
public:
	// New methods

	static CFolderView* NewL(CFolderPlayAppUi& aParent, RFs& aFs);

	virtual ~CFolderView();
	
	void SetRootL(const TDesC& anFocus = KNullDesC);
	void SetPathL(const TDesC& anPath, const TDesC& anFocus = KNullDesC);
	void SetVirtualPathL(const TDesC& anPath);
	TInt GetItemName(TDes& anName) const;
	void BackL();
	void SetPlayingItemL(const TDesC& anPlayingItem);
	HBufC16* GetFileContent(const TDesC& anFile);
	TInt GetTrackTimeL(const TDesC& anPath, TInt anIndex);
	
	void StorePlayStateL();
	void LoadPlayStateL();

	void ResumePause();
	void PlayResumePauseL();
	void Backward();
	void Forward();
	void ResetPlayList();
	void SyncStop(TBool aStore);

	void SeekStart(TInt aFFRwType);
	void SeekEnd();

	void DoSeekStart();
	void DoSeekEnd();

	void UpdatePlayPauseBtn();
	void PauseResumeRefresh();
	void StopRefresh();
	void StartRefresh();

	TBool PlayNextInList();

	void Start(TInt aPos);
	void Stop();
	void Pause();
	void Resume();

	void InformAniDll(TInt aCmd, TBool aPar);

	TBool IsDolbySupported();

	TBool IsFMTxPassive();

	void SetDolbyState(TBool aState);
	TBool GetDolbyState();

	void Refresh_LoadL();

	void StoreSettingsL();
	void LoadSettingsL();

	void FoldersScanningChangeL();
	void ShowHiddenFoldersChangeL();
	void UseTitleBarChangeL();

	void SetAlive(TBool aAlive);
	void SetTotalLength(TInt aTotalLength);
	TInt RecalcPos(TUint32 aSampleRate);

	void ChangeMP3DecodeType();

	void ChangeBoostVol();
	void ChangeVolumeBoost();
	void SetBoostVolVar();

protected:
	// from MMMFAudioResourceNotificationCallback
	virtual void MrccatoCommand(TRemConCoreApiOperationId aOperationId, TRemConCoreApiButtonAction aButtonAct);
	virtual void MrccatoPlay(TRemConCoreApiPlaybackSpeed aSpeed, TRemConCoreApiButtonAction aButtonAct);

	// Functions from base classes

	void SizeChanged();
	void FocusChanged(TDrawNow aDrawNow);
	TInt CountComponentControls() const;
	CCoeControl* ComponentControl(TInt aIndex) const;

	void HandleListBoxEventL(CEikListBox* aListBox, TListBoxEvent aEventType);
	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType);
	
	void GetPlayListL(const TDesC16& anPath);
	TBool IsSupported(const TDesC16& anName) const;
	TBool IsSupportedQ(const TDesC16& anName, TInt aExtPos, TBool& aCueMaybe) const;

	TBool SoundFileExistL(const TDesC16& anPath);
	
	void SetVolume(TInt aVol);
	
	void IncrementProgress(TInt aInc);
	TInt Progress() const; 

	void RefreshProgress(TBool aReset);
	void RefreshTitle();

	void AccesoryChangedAct(TInt aRestartTimeOut);

private:
	// Constructors

	void ConstructL();

	CFolderView(CFolderPlayAppUi& aParent, RFs& aFs);

	CDesC16ArrayFlat *GetSupportedHwrExtensionsL();

	void RefreshPos();
	static TInt FastForwardRewindPosUpdateCallback(TAny* aThis);

	void StartPosRefresh();
	static TInt PositionUpdateCallback(TAny* aThis);

	/**
	* From MCoeControlObserver. To handle the volume events.
	*/
	void HandleControlEventL( CCoeControl* aControl, TCoeEvent aEventType );

    /**
     * Callback for timer
     *
     * @param aPtr Pointer pass to this callback function.
     * @return Zero if callback function doesn't need to be called again.
     *         Otherwise, non-zero.
     */
    static TInt TimerCallback( TAny* aPtr );

    void UpdateVolume();

    // Equalizer Preset changed
    void EqPresetChanged(TInt aPreset);


	void ConnectedL(CAccMonitorInfo* aAccessoryInfo);
	void DisconnectedL(CAccMonitorInfo* aAccessoryInfo);
	void AccMonitorObserverError(TInt /*aError*/){};

    void AccesoryChangedL();
    static TInt TimerCallbackAccesory( TAny* aPtr );

	// from CDiskDetector::MObserver 
	void NotifyDiskChange(); 

    /**
     * From MBacklightControlObserver.
     * Callback for change in backlight status
     *
	 * @param aTarget The target of light change event; keypad, screen or both
	 * @param aStatus New status of the target
     */
    void LightStatusChanged( TInt aTarget, CHWRMLight::TLightStatus aStatus );

    TInt GetIconIdx(TBool aIsFolder);

protected:
	CFolderPlayAppUi& iParent;

	CAknTitlePane*                          iTitlePane;      // Not owned
	TInt					iDelayScrollTitle;

// GUI
	CAknSingleGraphicStyleListBox		iListBox;
	CMusicProgressBar*			iMusicProgress;

	CAknVolumePopup* iVolPopup;                     // owned
	CPeriodic* iTimer;                              // owned

	TBool	iIncreaseVol;
	TBool	iMuted;

// navigator	
	RFs&					iFs;

// special buttons	
	CRemConInterfaceSelector*		iRemSelector;
	CRemConKeyResponse*			iResponseHandler; // owned
	
	const CDesC16ArrayFlat*			iExtensions;
	const CDesC16ArrayFlat*			iInternalExts;

	CDesC16ArrayFlat*			iExtsForSearch;
	
	CCnvCharacterSetConverter*		iCharConv;


	TInt				iRepeat;
	TInt				iFFRwType;
	CPeriodic*			iFastForwardRewindPosUpdater;
	CPeriodic*			iPositionUpdater;


	CPlayEngine*			iPlayEngine;
	CSimpleAudioPlayer*		iInternalPlayer;

	CEqPresetNotifyHandler*		iEqPresetNotifier;

	CAccMonitor *iAccMonitor;

	CPeriodic*		iAccessoryTimer;         // owned
	TBool			iAccAtomarMode;
	TBool			iFirstTimeFlag;

	CDiskDetector* iDiskDetector; 

	CHWRMLight::TLightStatus iLightStatus;
	CHWRMLight *iLight;

public:
	TPath					iPath;

	CAknButton* iPlayPauseBtn;  // not own

	TInt iVolumeSteps;		// Number of volume steps in ui
	TInt iCurrentVol;

	TFoldPlayerType			iPlayerType;

	TInt				iTotalLength, iCurrentPos;

	TInt				iSeekPoint;
	TReal				iSeekRatio;

	TBool				iAlive, iPause;
	TBool				iSeeking;

	TBool				iIsRandom;
	TBool				iIsRepeatly;
	TBool				iIsRepeatlyOne;

	TBool				iInCueSheetMode;
	TBool				iRefreshTrackPosition;
	TBool				iNeedRefreshForPlayNextInList;

// play sequence	
	CDesC16ArrayFlat		iPlayList;
	TInt				iCurFileIdx;

	TBool				iDolbyActive;
	
	CAutoResumeHandler* iAutoResumeHandler;

	TBool				iFoldersScanning;

	TBool				iShowHiddingFolders;
	TUint				iAttribs;

	TBool				iUseTitleBar;

	TBool				iQuasiLameMP3;
	TBool				iQuasiLameMP3_Changed;

	TBool				iIsBoostVol;
	TBool				iBoostVolCur;
	TBool				iBoostVolChange;
	TInt				iBoostVal;

	friend class CFolderPlayAppUi;
};

#endif // __FOLDERPLAYVIEW_h__
// End of File
