// INCLUDE FILES
#include <flp_sdk_version.h>

#include <e32math.h>			// for Rand
#include <aknmessagequerydialog.h>
#include <e32property.h>
#include <featmgr.h>
#include <AknsUtils.h>

#include <mpxpskeywatcher.h>
#include <mpxequalizerview.h>

#include <hwrmfmtxdata.h>

#include <FolderPlay.rsg>
#include <FolderPlay.mbg>

#include <aknkeylock.h>

#include <versioninfo.h>  // VersionInfo

#include "FolderPlay.h"
#include "FolderPlayView.h"
#include "FolderPlayViewUi.h"


#define KFeatureIdKeypadNoVoiceKey   1014
#define KFeatureIdFmtx               1705

//    "" - !!?
#define KFeatureIdNoPowerkey         1639

//  Properties
static _LIT_SECURITY_POLICY_PASS(KAllowAllPolicy);

_LIT( KAnimDllFilename, "SrvcA000A727.dll" );

// FM Transmitter status key ID of the FM TX property.
// Note: This data is interpreted as TFmTxState below
const TUid KPSUidHWRMFmTx   = { 0x101F7A02 }; // FM TX category ID
const TUint32 KHWRMFmTxStatus  = 0x00000001;

const TInt KKeyPadLockStatus = 1002; // For Inform Anim Dll - dsv
const TInt KMenuKeySimulateCancel = 1004;
const TInt KCaptureHwrKeys   = 1005;
const TInt KIsBelle_FP2 = 1006;

const TChar KBackSlash = '\\';

enum TAniDllResponse
{
  EaDllResponse_None = 0,
  EaDllResponse_RedKey = 1,
  EaDllResponse_GreenKey = 2,
  EaDllResponse_MenuKey = 4,
  EaDllResponse_MenuKeyLongTime = 8,
  EaDllResponse_VoiceCmdKey = 0x10,
  EaDllResponse_CameraKey = 0x20,
  EaDllResponse_FocusKey = 0x40
};

/////////////////////////////////////////////

TInt GetS60PlatformVersionL(RFs& aFs, TUint& aMajor, TUint& aMinor)
{
    VersionInfo::TPlatformVersion platformVersion;
    TInt ret = VersionInfo::GetVersion(platformVersion, aFs);
    if (ret == KErrNone)
    {
        aMajor = platformVersion.iMajorVersion;
        aMinor = platformVersion.iMinorVersion;
    }
    return ret;
}

void CFolderPlayAppUi::ConstructL()
{
#ifdef _FLP_SDK_VERSION_BELLE_
	BaseConstructL(CAknAppUi::EAknEnableSkin | EAknTouchCompatible | EAknSingleClickCompatible);
#else
	BaseConstructL(CAknAppUi::EAknEnableSkin);
#endif

	User::LeaveIfError(iFs.Connect()); iFs.ShareAuto();
	iFs.CreatePrivatePath(EDriveC);

        FeatureManager::InitializeLibL();
        iDolbySupported = FeatureManager::FeatureSupported( 0x19D ); // DolbyHeadphone

	iIsVoiceKey = !(FeatureManager::FeatureSupported( KFeatureIdKeypadNoVoiceKey ));

	iFM_TX_Supported = FeatureManager::FeatureSupported( KFeatureIdFmtx );

	CAknButton* aPlayPauseBtn(NULL);

	iToolBar = CurrentFixedToolbar();
	if(iToolBar)
	{
		iToolBar->SetToolbarVisibility(ETrue);
		iToolBar->SetToolbarObserver(this);
		aPlayPauseBtn = static_cast<CAknButton*>( iToolBar->ControlOrNull( EPlay ) );
		aPlayPauseBtn->SetCurrentState(1, ETrue);
	}

	iAppView = CFolderView::NewL(*this, iFs);
	iAppView->iPlayPauseBtn = aPlayPauseBtn;
	iAppView->SetMopParent(this);
	AddToStackL(iAppView);

	//  RProperty    AnimDLL
	User::LeaveIfError(RProperty::Define(KUidFolderPlayApp, 1, RProperty::EInt, KAllowAllPolicy, KAllowAllPolicy));

	//     
	RWindowBase* windowBase = iAppView->DrawableWindow();

	iClientDll = RAnimDll(iWs);
	User::LeaveIfError( iClientDll.Load( KAnimDllFilename ) );

	iSblAnim = new ( ELeave ) RSblAnim( iClientDll );
	User::LeaveIfError( iSblAnim->Construct( *windowBase ) );

	iKeyObserver = CMPXPSKeyWatcher::NewL(KUidFolderPlayApp, 1,this);


        iEqView = CMPXEqualizerView::NewL();

	User::LeaveIfError(CEikonEnv::Static()->RootWin().EnableGroupListChangeEvents());

	SendBoolCmdToAnimDll(KCaptureHwrKeys, iCaptureHwrKeys);

// Obtain Software Platform version number
	TUint MajorVer(0);
	TUint MinorVer(0);

	GetS60PlatformVersionL(iFs, MajorVer, MinorVer);

	SendBoolCmdToAnimDll(KIsBelle_FP2, MajorVer >= 5 && MinorVer >= 5);

	if(iFM_TX_Supported)
	{
	   iFM_TX_KeyObserver = CMPXPSKeyWatcher::NewL(KPSUidHWRMFmTx, KHWRMFmTxStatus,this);

	   GetFmTxState();
	}
}

CFolderPlayAppUi::CFolderPlayAppUi(RWsSession& aWs)
 :iWs( aWs )
{
	iAppView = NULL;
	iDolbySupported = EFalse;
	iDolbyState = EFalse;
}


CFolderPlayAppUi::~CFolderPlayAppUi()
{
	delete iEqView;

/////////
	if(iKeyObserver) delete iKeyObserver;

	if (iSblAnim)
	{
		iSblAnim->Close();
		delete iSblAnim;
	}
	iClientDll.Close();

	RProperty::Delete(KUidFolderPlayApp, 1);
/////////

	if(iFM_TX_KeyObserver) delete iFM_TX_KeyObserver;

	if(iAppView)
	{
		RemoveFromStack(iAppView);
		delete iAppView;
	}

        FeatureManager::UnInitializeLib();

	iFs.Close();
}


void CFolderPlayAppUi::HandleCommandL(TInt aCommand)
{
	TFileName aName;

	TBool aRepeatly(EFalse);

	switch (aCommand)
	{
		case EPlay_Repeatly:
			aRepeatly = ETrue;
		case EPlay_Siblings:  // Play sequentially
			if(iAppView->iAlive) iAppView->SyncStop(EFalse);
		case EPlay:
		{	
			TInt aType = iAppView->GetItemName(aName);

			if(iAppView->iAlive || aType != EMbmFolderplayFolder)
			{
				iAppView->ResumePause();
			}
			else
			{
				iAppView->ResetPlayList();

				iAppView->iIsRepeatly = aRepeatly;

				aName.Insert(0, iAppView->iPath);
				aName.Append(KBackSlash);
				iAppView->GetPlayListL(aName);
				iAppView->Start(0);
			}
		} break;
			
		case EPlay_This_Repeatly:
			aRepeatly = ETrue;
		case EPlay_This:
		{	
			TInt aType = iAppView->GetItemName(aName);

			if(aType == EMbmFolderplayFile || aType == EMbmFolderplayVirtual)
			{
				if(iAppView->iAlive) iAppView->SyncStop(EFalse);

				iAppView->ResetPlayList();

				iAppView->iIsRepeatlyOne = aRepeatly;
				
				aName.Insert(0, iAppView->iPath);
				iAppView->iPlayList.AppendL(aName);
				iAppView->Start(0);
			}
		} break;

		case EPlay_Shuffled:
		{
			if(iAppView->GetItemName(aName) != EMbmFolderplayFolder) break;

			if(iAppView->iAlive) iAppView->SyncStop(EFalse);
			
			iAppView->ResetPlayList();

			aName.Insert(0, iAppView->iPath);
			aName.Append(KBackSlash);

			iAppView->GetPlayListL(aName);
			if(iAppView->iPlayList.Count() == 0) break;

			TTime aNow; aNow.HomeTime();
			TInt64 aSeed = aNow.Int64();

			TInt n = iAppView->iPlayList.Count();
			while(--n > 0) 
			{
				TInt k = Math::Rand(aSeed) % (n + 1);		// 0 <= k <= n
				if(k != n) Mem::Swap(iAppView->iPlayList.At(k), iAppView->iPlayList.At(n), iAppView->iPlayList.Length());
			}
			iAppView->iIsRandom = ETrue;
			iAppView->Start(0);
		} break;
		
		case EBackward: iAppView->Backward(); break;
			
		case EForward: 	iAppView->Forward(); break;
				
		case EAknSoftkeyBack:
			if(iAppView->iPath.Length() == 0)
			{
				TApaTask task(iWs);
				task.SetWgId( CEikonEnv::Static()->RootWin().Identifier());
				task.SendToBackground();
			}
			else	iAppView->BackL();
			break;
		
		case ERefresh_Load:
			iAppView->Refresh_LoadL();
			break;
		
		case EStop:
			iAppView->SyncStop(ETrue);
			break;
		
		case EEikCmdExit:
		case EAknSoftkeyExit:
			if(iAppView->iAlive) iAppView->SyncStop(ETrue);
			iAppView->StoreSettingsL();
			Exit();
			break;

		case ESeekForward:
			iAppView->SeekStart(EFFType);
			break;

		case ESeekBackward:
			iAppView->SeekStart(ERwType);
			break;

		case EEndSeek:
			iAppView->SeekEnd();
			break;

	case EAbout:
		DisplayDialog(R_ABOUT_DIALOG_TITLE, R_ABOUT_DIALOG_TEXT);
		break;

	case EDolbyHeadphone:
		iDolbyState = !iDolbyState;
		iAppView->SetDolbyState(iDolbyState);
            break;

	case ESettingSubMenuCaptureHwrKeys:
		iCaptureHwrKeys = !iCaptureHwrKeys;
		SendBoolCmdToAnimDll(KCaptureHwrKeys, iCaptureHwrKeys);
            break;

	case ESettingSubMenuScanFolder:
		iAppView->FoldersScanningChangeL();
            break;

	case ESettingSubMenuDisplayHiddenFolder:
		iAppView->ShowHiddenFoldersChangeL();
            break;

	case ESettingSubMenuUseTitleBar:
		iAppView->UseTitleBarChangeL();
            break;

	case EQuasiGaplessMP3:
		iAppView->ChangeMP3DecodeType();
            break;

	case EBoostVol:
		iAppView->ChangeBoostVol();
            break;

        case ECmdEqualizer:
            {
	    iEqView->LaunchEqualizerL();
            break;
            }

	default: CAknAppUi::HandleCommandL(aCommand); break;
	}
}


void CFolderPlayAppUi::OfferToolbarEventL(TInt aCommand)
{
    TInt commandId = aCommand;

    if(AknLayoutUtils::PenEnabled())
    {
    TInt eventModifiers = iToolBar->EventModifiers();
    if ( eventModifiers == CAknToolbar::ELongPress )
        {
        if ( aCommand == EForward )
            {
            commandId = ESeekForward;
            }
        else if ( aCommand == EBackward )
            {
            commandId = ESeekBackward;
            }
        else if ( aCommand == EPlay )
            {
		if (iAppView->iAlive)
		  commandId = EStop;
		else
		  commandId = ERefresh_Load;
            }
        }
    else if ( eventModifiers == CAknToolbar::ELongPressEnded )
        {
        if ( aCommand == EForward || aCommand == EBackward )
            {
            commandId = EEndSeek;
            }
        else if ( aCommand == EPlay )
            {
	        commandId = EDummy;
		iAppView->UpdatePlayPauseBtn();
	    }
        }
    }

	HandleCommandL(commandId);
}


void CFolderPlayAppUi::HandleWsEventL(const TWsEvent& aEvent, CCoeControl* aDestination)
{
	if(aEvent.Type() == EEventWindowGroupListChanged)
	{
		RAknKeylock2 keyLock;
		TInt err;
		TRAP(err, keyLock.Connect());
		if(err == KErrNone)
		{
			TBool locked = keyLock.IsKeyLockEnabled();

			SendBoolCmdToAnimDll(KKeyPadLockStatus, locked);

			keyLock.Close();
		}
	}

	if(aEvent.Type() != KAknUidValueEndKeyCloseEvent)
		CAknAppUi::HandleWsEventL(aEvent, aDestination);
}


void CFolderPlayAppUi::DynInitMenuPaneL(TInt aResourceId, CEikMenuPane* aMenuPane)
{
	if( aResourceId == R_MENU )
	{
		TFileName aName;
		TInt aType = iAppView->GetItemName(aName);

		TBool aDimmedFolderItem(EFalse);
		TBool aDimmedFileItem(EFalse);

		switch(aType)
		{
		case EMbmFolderplayFolder:
			aDimmedFolderItem = EFalse;
			aDimmedFileItem   = ETrue;
		break;
					
		case EMbmFolderplayFile:
		case EMbmFolderplayVirtual:
			aDimmedFolderItem = ETrue;
			aDimmedFileItem   = EFalse;
		break;
				
		default:
			aDimmedFolderItem = ETrue;
			aDimmedFileItem   = ETrue;
		}

		aMenuPane->SetItemDimmed( EPlay_Shuffled, aDimmedFolderItem);
		aMenuPane->SetItemDimmed( EPlay_Siblings, aDimmedFolderItem);
		aMenuPane->SetItemDimmed( EPlay_Repeatly, aDimmedFolderItem);

		aMenuPane->SetItemDimmed( EPlay_This, aDimmedFileItem);
		aMenuPane->SetItemDimmed( EPlay_This_Repeatly, aDimmedFileItem);
	}
	else if(aResourceId == R_SETTINGS_SUB_MENU)
	{
		aMenuPane->SetItemButtonState( ESettingSubMenuScanFolder, 
					iAppView->iFoldersScanning ?
					EEikMenuItemSymbolOn :
					EEikMenuItemSymbolIndeterminate );

		aMenuPane->SetItemButtonState( ESettingSubMenuDisplayHiddenFolder, 
					iAppView->iShowHiddingFolders ?
					EEikMenuItemSymbolOn :
					EEikMenuItemSymbolIndeterminate );

		aMenuPane->SetItemButtonState( ESettingSubMenuUseTitleBar, 
					iAppView->iUseTitleBar ?
					EEikMenuItemSymbolOn :
					EEikMenuItemSymbolIndeterminate );

		aMenuPane->SetItemButtonState( ESettingSubMenuCaptureHwrKeys, 
					iCaptureHwrKeys ?
					EEikMenuItemSymbolOn :
					EEikMenuItemSymbolIndeterminate );
	}
	else if(aResourceId == R_AUDIO_SUB_MENU)
	{
	      if(iDolbySupported && iAppView->IsFMTxPassive())
	      {
		aMenuPane->SetItemDimmed( EDolbyHeadphone, EFalse);

		TInt aPos;
		CEikMenuPaneItem* aMenuPanePos = aMenuPane->ItemAndPos( EDolbyHeadphone, aPos);

		if(iDolbyState && iAppView->iDolbyActive)
		{
//			TPtrC KMBMFile = _L("\\resource\\apps\\FolderPlay.mbm");
//			CGulIcon* aDolbyIcon = iEikonEnv->CreateIconL(KMBMFile, EMbmFolderplayDolbyheadphone, EMbmFolderplayDolbyheadphone_mask);

			MAknsSkinInstance* skin = AknsUtils::SkinInstance();
			TInt err(KErrNone);
                        CGulIcon* aDolbyIcon(NULL);

			// EAknsMinorGenericQtgGrafDolbyHeadphone = 0x293e;
			static const TAknsItemID KAknsIIDQtgGrafDolbyHeadphone_ = { EAknsMajorGeneric, 0x293e };

			TRAP(err,
			     aDolbyIcon = AknsUtils::CreateGulIconL(
			                skin,
			                KAknsIIDQtgGrafDolbyHeadphone_,
					ETrue // RequireMask
					));

			if(err == KErrNone) aMenuPanePos->SetIcon(aDolbyIcon);
		}
		else
		{
			aMenuPanePos->SetIcon(NULL);

			aMenuPane->SetItemButtonState( EDolbyHeadphone, 
					iDolbyState ?
					EEikMenuItemSymbolOn :
					EEikMenuItemSymbolIndeterminate );
		}
	      }
	      else
	      {
		aMenuPane->SetItemDimmed( EDolbyHeadphone,  ETrue);
	      }

	      aMenuPane->SetItemButtonState( EQuasiGaplessMP3, 
				iAppView->iQuasiLameMP3 ?
				EEikMenuItemSymbolOn :
				EEikMenuItemSymbolIndeterminate );

	      if (iAppView->iPlayerType != EFFPtExternal)
	      {
		aMenuPane->SetItemDimmed( EBoostVol,  EFalse);

		aMenuPane->SetItemButtonState( EBoostVol, 
				iAppView->iIsBoostVol ?
				EEikMenuItemSymbolOn :
				EEikMenuItemSymbolIndeterminate );
	      }
	      else
		aMenuPane->SetItemDimmed( EBoostVol,  ETrue);
	}
}


void CFolderPlayAppUi::HandleStatusPaneSizeChange()
{
	iAppView->SetRect(ClientRect());
}


void CFolderPlayAppUi::GetFmTxState()
{
         TInt value(EFmTxStateUnknown);

         if( iFM_TX_KeyObserver->GetValue( value ) == KErrNone )
         {
           // EFmTxStatePowerSaveAccessory are considered as off
	   iFmTxActivity = (TBool)(value == EFmTxStateActive || value == EFmTxStateInactive || value == EFmTxStatePowerSaveInactivity);
         }
         else iFmTxActivity = EFalse;
}


void CFolderPlayAppUi::HandlePSEvent(TUid aUid, TInt aKey)
{
	if(iFM_TX_Supported && aUid == KPSUidHWRMFmTx)
	{
	   GetFmTxState();

	   if(iFmTxActivity) iAppView->AccesoryChangedAct(1500000);
	}
	else if(aUid == KUidFolderPlayApp)
	{
	    TInt aVal(0);
        
	    if(!iKeyObserver->GetValue(aVal))
	    {
		switch(aVal)
		{
		case EaDllResponse_VoiceCmdKey:
		 {
		   if(iIsVoiceKey) iAppView->PlayResumePauseL();
		 } break;

		case EaDllResponse_RedKey:
		 {
		   iAppView->Forward();
		 } break;

		case EaDllResponse_GreenKey:
		 {
		   iAppView->Backward();
		 } break;

		case EaDllResponse_MenuKey:
		 {
		   if(iAppView->iAlive)	iAppView->ResumePause();
		 } break;

		case EaDllResponse_MenuKeyLongTime:
		 {
		   if(iAppView->iAlive)
		     iAppView->SyncStop(ETrue); //store!!!
		   else
		     iAppView->Refresh_LoadL();
		 } break;

		case EaDllResponse_CameraKey:
		 {
		   if(iAppView->iAlive) iAppView->Forward();
		 } break;

		//Simulating menu key down
		case EStdKeyMenu: //Menu key
		case EStdKeyApplication0:
		 {
		    TRawEvent event;

		    event.Set(TRawEvent::EKeyDown, aVal);
		    iWs.SimulateRawEvent(event);

		    event.Set(TRawEvent::EKeyUp, aVal);
		    iWs.SimulateRawEvent(event);

		    iWs.Flush();

		    SendBoolCmdToAnimDll(KMenuKeySimulateCancel, EFalse);

		 } break;

		default:
		  break;
		}
	    }
	}
}


//  AnimDll     TBool
void CFolderPlayAppUi::SendBoolCmdToAnimDll(TInt aCmd, TBool aPar)
{
  TPckgC<TBool> pars(aPar);

  iSblAnim->AnimCmd(aCmd, pars);

  iWs.Flush();
}


void CFolderPlayAppUi::DisplayDialog(TInt aTitle, TInt aText)
{
	CAknMessageQueryDialog* dlg = new (ELeave) CAknMessageQueryDialog();
	dlg->PrepareLC(R_ABOUT_QUERY_DIALOG);
	HBufC* title = iEikonEnv->AllocReadResourceLC(aTitle);
	dlg->QueryHeading()->SetTextL(*title);
	CleanupStack::PopAndDestroy(); //title
	HBufC* msg = iEikonEnv->AllocReadResourceLC(aText);
	dlg->SetMessageTextL(*msg);
	CleanupStack::PopAndDestroy(); //msg
	dlg->RunLD();
}


// End of File
