// INCLUDE FILES
#include <AknsUtils.h>
#include <AknUtils.h>
#include <AknsDrawUtils.h>
#include <aknlayout.cdl.h>
#include <AknsBasicBackgroundControlContext.h>
#include <eiklabel.h>
#include <gulicon.h>
#include <touchfeedback.h>
#include <StringLoader.h>

#include <avkon.rsg>
#include <mpxcommonui.rsg>

#include "musicprogressbar.h"
#include "FolderPlayView.h"


// CONSTANTS
//_LIT(KMPXCommonPlaybackViewIconFile, "\\resource\\apps\\mpxcommonplaybackview.mif");

// delay for actual seek operation
const TInt KSeekDelay = 100000; // 0.1s

const TInt KOneHourInSeconds = 3600;
const TInt KMPXOneHundredHoursInSecs = 360000;
const TInt KMPXOneSecInMicroSecs = 1000000;
const TInt KMPXDurationDisplayResvLen = 10;

HBufC* CMusicProgressBar::DisplayableDurationL( TInt64 aDuration )
{
    TInt resId;
    HBufC* format = NULL;
    TInt64 Duration(aDuration / 1000);

    if ( Duration >= KMPXOneHundredHoursInSecs )
    {
        resId = R_MPX_TIME_DURAT_HHHMMSS_WITH_ZERO;
        format = StringLoader::LoadLC( resId, Duration / KOneHourInSeconds );
    }
    else
    {
        resId = R_QTN_TIME_DURAT_MIN_SEC_WITH_ZERO;
        if (( iMode == EMPXDuratHMS ) || ( Duration >= KOneHourInSeconds ))
        {
           resId = R_QTN_TIME_DURAT_LONG_WITH_ZERO;
        }
        format = StringLoader::LoadLC( resId );
    }

    // Convert total playing time to texts.
    TTime durTime( Duration * KMPXOneSecInMicroSecs );

    HBufC* buf = HBufC::NewLC( format->Length() + KMPXDurationDisplayResvLen );
    TPtr bufTPtr( buf->Des() );

    durTime.FormatL( bufTPtr, *format );
    AknTextUtils::LanguageSpecificNumberConversion( bufTPtr );
    CleanupStack::Pop( buf );
    CleanupStack::PopAndDestroy( format );

    return buf;
}

// ======== MEMBER FUNCTIONS ========

CMusicProgressBar* CMusicProgressBar::NewL( const TRect& aRect, const CCoeControl* aParent )
{
    CMusicProgressBar* self = new ( ELeave ) CMusicProgressBar;
    CleanupStack::PushL( self );
    self->ConstructL( aRect, aParent );
    CleanupStack::Pop( self );
    return self;
}


void CMusicProgressBar::ConstructL( const TRect& aRect, const CCoeControl* aParent )
{
    SetContainerWindowL( *aParent );

    iParent = (CFolderView*)aParent;

    iAlive = EFalse;

    iTouchDown = EFalse;
    iIsForeground = EFalse;

    iMode = EMPXDuratAuto;

    iSeekTimer = CPeriodic::NewL( EPriorityLow );

    TAknWindowLineLayout screenLayout = AknLayout::screen();

    TRect screenRect = screenLayout.Rect();
    iBackground = CAknsBasicBackgroundControlContext::NewL(
			KAknsIIDQsnBgAreaMain, screenRect, EFalse );

    // Create labels

    iTextElapsedTime = new ( ELeave ) CEikLabel();
    CleanupStack::PushL( iTextElapsedTime );
    iTextElapsedTime->SetContainerWindowL( *this );
    iTextElapsedTime->SetTextL( KNullDesC );
    CleanupStack::Pop( iTextElapsedTime );

    iTextRemainingTime = new ( ELeave ) CEikLabel();
    CleanupStack::PushL( iTextRemainingTime );
    iTextRemainingTime->SetContainerWindowL( *this );
    iTextRemainingTime->SetTextL( KNullDesC );
    CleanupStack::Pop( iTextRemainingTime );

    iTextTotalTime = new ( ELeave ) CEikLabel();
    CleanupStack::PushL( iTextTotalTime );
    iTextTotalTime->SetContainerWindowL( *this );
    iTextTotalTime->SetTextL( KNullDesC );
    CleanupStack::Pop( iTextTotalTime );

    CreateIconsL();

    SetRect( aRect );

    TRAP_IGNORE( DoUpdateLayoutL() );

    DrawableWindow()->SetPointerGrab( ETrue );
    EnableDragEvents();

    ActivateL();
}


CMusicProgressBar::~CMusicProgressBar()
{
    // Pointer events
    SetGloballyCapturing( EFalse );
    SetPointerCapture( EFalse );

    FreeIcons();

    delete iBackground;

    delete iTextElapsedTime;   //dsv
    delete iTextRemainingTime; //dsv
    delete iTextTotalTime;     //dsv

    MTouchFeedback* feedback = MTouchFeedback::Instance();
    if (feedback) feedback->RemoveFeedbackForControl(this);

    if ( iSeekTimer )
    {
      iSeekTimer->Cancel();
      delete iSeekTimer;
    }
}


void CMusicProgressBar::UpdateTimeIndicatorsL( TInt aPlaybackPos, TInt aTotalLength )
{
   iTrackTotalLength = aTotalLength;
   iNewSongPosition = aPlaybackPos;

   iMode = EMPXDuratAuto;
   if ( iTrackTotalLength / 1000 > KOneHourInSeconds ) iMode = EMPXDuratHMS;

   if (!iTouchDown)
   {
    if ( aTotalLength == 0 || iHideTimeIndicators)
    {
        UpdateLabelL( iTextElapsedTime, KNullDesC );
        UpdateLabelL( iTextRemainingTime, KNullDesC );
        UpdateLabelL( iTextTotalTime, KNullDesC );
    }
    else
    {
        // Convert ellapsed time to texts
        HBufC* position = DisplayableDurationL( aPlaybackPos );
        CleanupStack::PushL( position );

        TInt remainingTime = aTotalLength - aPlaybackPos;
        if ( remainingTime < 0 ) remainingTime = 0; // Convert all negative value to zero

        // Convert remaining time to texts
        HBufC* remaining = DisplayableDurationL( remainingTime );
        CleanupStack::PushL( remaining );

        HBufC* total = DisplayableDurationL(iTrackTotalLength);
        CleanupStack::PushL( total );

        UpdateLabelL( iTextElapsedTime, *position );
        UpdateLabelL( iTextRemainingTime, *remaining );
        UpdateLabelL( iTextTotalTime, *total );

        CleanupStack::PopAndDestroy( total );
        CleanupStack::PopAndDestroy( remaining );
        CleanupStack::PopAndDestroy( position );
    }

    if ( aTotalLength )
    {
       TReal t = aTotalLength;
       iPlaybackRatio = aPlaybackPos / t;
    }
    else
       iPlaybackRatio = 0;

    RefreshProgressBar();
   }
}


void CMusicProgressBar::UpdateLabelL( CEikLabel* aLabel, const TDesC& aText )
{
    if ( aLabel )
    {
       if ( *(aLabel->Text()) != aText )
       {
          aLabel->SetTextL(aText);
          aLabel->DrawDeferred();
       }
    }
}


void CMusicProgressBar::SetForegroundL( TBool aForeground )
{
    iIsForeground = aForeground;

    if ( !aForeground && iTouchDown )
    {
       // Pointer events
       SetGloballyCapturing( EFalse );
       SetPointerCapture( EFalse );

       iTouchDown = EFalse;
    }

    RefreshProgressBar();
}


// ---------------------------------------------------------------------------
// From CCoeControl
// Handles a change to the control's resources.
// ---------------------------------------------------------------------------
void CMusicProgressBar::HandleResourceChange( TInt aType )
{
    CCoeControl::HandleResourceChange( aType );

    if ( aType == KAknsMessageSkinChange )
    {
        TRAP_IGNORE(
            {
            CreateIconsL();    // Reload icons
            DoUpdateLayoutL(); // Make sure execute layout again
            } );
    }
}


// ---------------------------------------------------------------------------
// Creates icons.
// ---------------------------------------------------------------------------
void CMusicProgressBar::CreateIconsL()
{
    FreeIcons();

    TFileName iconFile( KMPXCommonPlaybackViewIconFile );

    MAknsSkinInstance* skin = AknsUtils::SkinInstance();

    iPlaybackSlider = AknsUtils::CreateGulIconL(
                skin,
                KAknsIIDQgnGrafMup2BarProgress2,
                iconFile,
                16396, //EMbmMpxcommonplaybackviewQgn_graf_mup_npv_prbar_top_play
                16397  //EMbmMpxcommonplaybackviewQgn_graf_mup_npv_prbar_top_play_mask
		);

    iSliderBackground = AknsUtils::CreateGulIconL(
                skin,
                KAknsIIDQgnGrafMup2BarFrame,
                iconFile,
                16394, //EMbmMpxcommonplaybackviewQgn_graf_mup_npv_prbar_bgd
                16395  //EMbmMpxcommonplaybackviewQgn_graf_mup_npv_prbar_bgd_mask
		);
}


void CMusicProgressBar::FreeIcons()
{
    delete iPlaybackSlider;
    iPlaybackSlider = NULL;

    delete iSliderBackground;
    iSliderBackground = NULL;
}

// ---------------------------------------------------------------------------
// Draw an icon
// ---------------------------------------------------------------------------
void CMusicProgressBar::DrawIndicator(
    CWindowGc& aGc ,
    const TRect& aDirtyRect,
    const TRect& aIndicatorRect,
    const CGulIcon* aIcon,
    TBool aInvertMask ) const
{
    if ( aIcon && aIndicatorRect.Intersects( aDirtyRect ) )
    {
       aGc.BitBltMasked(
           aIndicatorRect.iTl,
           aIcon->Bitmap(),
           TRect( aIndicatorRect.Size() ),
           aIcon->Mask(),
           aInvertMask );
    }
}


// ---------------------------------------------------------------------------
// Update progress bar graphics based on current playback
// ---------------------------------------------------------------------------
void CMusicProgressBar::UpdateProgressBarGraphics()
{
    TSize pbBarSize = iPlaybackSliderRect.Size();

    pbBarSize.iWidth *= iPlaybackRatio;

    AknIconUtils::SetSize( iPlaybackSlider->Bitmap(), pbBarSize, EAspectRatioNotPreserved );
}


void CMusicProgressBar::RefreshProgressBar()
{
    if ( iIsForeground )
    {
      UpdateProgressBarGraphics();
      Window().Invalidate( iSliderPaneRect );
    }
}


// ---------------------------------------------------------------------------
// From CCoeControl
// Responds to changes to the size and position of the contents of this
// control.
// ---------------------------------------------------------------------------
void CMusicProgressBar::SizeChanged()
{
    TRAP_IGNORE( DoUpdateLayoutL() );

    CAknsBasicBackgroundControlContext* background =
        static_cast<CAknsBasicBackgroundControlContext*>( iBackground );

    background->SetBitmap( KAknsIIDQsnBgAreaMain );
    background->SetRect( Rect() );
}

// ---------------------------------------------------------------------------
// From CCoeControl
// Gets the number of controls contained in a compound control.
// ---------------------------------------------------------------------------
TInt CMusicProgressBar::CountComponentControls() const
{
     return 3;
}

// ---------------------------------------------------------------------------
// From CCoeControl
// Gets an indexed component of a compound control.
// ---------------------------------------------------------------------------
CCoeControl* CMusicProgressBar::ComponentControl( TInt aIndex ) const
{
   switch(aIndex)
   {
   case 0:
     return (CCoeControl*)iTextElapsedTime;

   case 1:
     return (CCoeControl*)iTextRemainingTime;

   case 2:
     return (CCoeControl*)iTextTotalTime;

   default:
     return NULL;
   }
}

// ---------------------------------------------------------------------------
// From CCoeControl. Draws the control.
// ---------------------------------------------------------------------------
void CMusicProgressBar::Draw( const TRect& aRect ) const
{
    CWindowGc& gc = SystemGc();
    MAknsSkinInstance* skin = AknsUtils::SkinInstance();

    AknsDrawUtils::Background( skin, iBackground, this, gc, aRect );

    DrawIndicator( gc, aRect, iSliderBackgroundRect, iSliderBackground );

    if( iAlive )
    {
      DrawIndicator(gc, aRect, TRect(iPlaybackSliderRect.iTl,
                   iPlaybackSlider->Bitmap()->SizeInPixels() ),
                   iPlaybackSlider );
    }
}


void CMusicProgressBar::HandlePointerEventL(const TPointerEvent& aPointerEvent)
{
    TSize pbBarSize = iPlaybackSliderRect.Size();
    TReal width;

    switch( aPointerEvent.iType )
    {
    case TPointerEvent::EButton1Down:
        if ( iSliderPaneRect.Contains(aPointerEvent.iPosition) && iAlive )
        {
            // Pointer events
            SetGloballyCapturing( ETrue );
            SetPointerCapture( ETrue );

            iParent->DoSeekStart();

            iTouchDown = ETrue;
            width = aPointerEvent.iPosition.iX - iSliderBackgroundRect.iTl.iX;
            pbBarSize.iWidth = width;
            AknIconUtils::SetSize( iPlaybackSlider->Bitmap(), pbBarSize, EAspectRatioNotPreserved );
        }
        break;

    case TPointerEvent::EDrag:
        if(iTouchDown && iAlive )
        {
            MTouchFeedback* feedback = MTouchFeedback::Instance();
            if (feedback) feedback->InstantFeedback(ETouchFeedbackSensitive);

            if (aPointerEvent.iPosition.iX <= iSliderBackgroundRect.iTl.iX)
            {
              width = 0;
            }
            else if (aPointerEvent.iPosition.iX >= iSliderBackgroundRect.iBr.iX)
            {
              width = iSliderBackgroundRect.iBr.iX - iSliderBackgroundRect.iTl.iX;
            }
            else
            {
              width = aPointerEvent.iPosition.iX - iSliderBackgroundRect.iTl.iX;
            }

            pbBarSize.iWidth = width;
            AknIconUtils::SetSize( iPlaybackSlider->Bitmap(), pbBarSize, EAspectRatioNotPreserved );

            // Update the ellapsed and remaining time texts for user feedback
            TReal totalWidth = iPlaybackSliderRect.Width();
            TReal ratio = width/totalWidth;
            TInt newSongPosition = ratio * iTrackTotalLength;

            if (!iHideTimeIndicators)
            {
             // Convert ellapsed time to text
             HBufC* position = DisplayableDurationL( newSongPosition );
             CleanupStack::PushL( position );

             TInt remainingTime = iTrackTotalLength - newSongPosition;
             if ( remainingTime < 0 ) remainingTime = 0;

             // Convert remaining time to text
             HBufC* remaining = DisplayableDurationL( remainingTime );
             CleanupStack::PushL( remaining );

             UpdateLabelL( iTextElapsedTime, *position );
             UpdateLabelL( iTextRemainingTime, *remaining );
             CleanupStack::PopAndDestroy( remaining );
             CleanupStack::PopAndDestroy( position );
            }

            // Invalidate rect so it will be redrawn immediately
            Window().Invalidate( iSliderPaneRect );
        }
        else
        {
            SetGloballyCapturing( EFalse );
            SetPointerCapture( EFalse );
            iTouchDown = EFalse;
        }
        break;

    case TPointerEvent::EButton1Up:
        if(iTouchDown)
        {
            // Pointer events
            SetGloballyCapturing( EFalse );
            SetPointerCapture( EFalse );

            if ( iAlive )
            {
              iTouchDown = EFalse;

              if ( aPointerEvent.iPosition.iX <= iSliderBackgroundRect.iTl.iX )
              {
                width = 0;
              }
              else if ( aPointerEvent.iPosition.iX >= iSliderBackgroundRect.iBr.iX )
              {
                width = iSliderBackgroundRect.iBr.iX - iSliderBackgroundRect.iTl.iX;
              }
              else
              {
                width = aPointerEvent.iPosition.iX - iSliderBackgroundRect.iTl.iX;
              }

              pbBarSize.iWidth = width;
              AknIconUtils::SetSize( iPlaybackSlider->Bitmap(), pbBarSize, EAspectRatioNotPreserved );

              // Set new song position based upon progress bar location
              TReal totalWidth = iPlaybackSliderRect.Width();
              TReal ratio = width / totalWidth;
              iNewSongPosition = (ratio * iTrackTotalLength);

              // Invalidate rect so it will be redrawn immediately
              Window().Invalidate( iSliderPaneRect );
              iSeekTimer->Cancel();
              iSeekTimer->Start( KSeekDelay, KSeekDelay, TCallBack( CallSetPosition, this ) );
            }
        }
        break;

    default:;
    }
}


void CMusicProgressBar::SetAlive( TBool aAlive )
{
    iAlive = aAlive;
}


TInt CMusicProgressBar::GetNewSongPosition()
{
    return iNewSongPosition;
}


// ---------------------------------------------------------------------------
// Helper function to set layout for all elements and set sizes for icons.
// ---------------------------------------------------------------------------
void CMusicProgressBar::DoUpdateLayoutL()
{
    iSliderPaneRect = Rect(); // dsv
    iSliderBackgroundRect = iSliderPaneRect; // dsv

    iPlaybackSliderRect.iTl.iX = iSliderPaneRect.iTl.iX + 3; // dsv
    iPlaybackSliderRect.iTl.iY = iSliderPaneRect.iTl.iY + 1; //
    iPlaybackSliderRect.iBr.iX = iSliderPaneRect.iBr.iX - 2; //
    iPlaybackSliderRect.iBr.iY = iSliderPaneRect.iBr.iY - 1; //

    //ETextElapsedTime
    AknLayoutUtils::LayoutLabel( iTextElapsedTime,
				 iSliderPaneRect,
				 EAknLogicalFontPrimarySmallFont,//EAknLogicalFontSecondaryFont,
				 0,                         // C colour index
				 6,                         // l left margin
				 88,                        // r right margin
				 30,                        // B Baseline from top of the parent rectangle
				 82,                        // W text width in pixels
				 ELayoutAlignLeft );        // J justification

    //ETextRemainingTime
    AknLayoutUtils::LayoutLabel( iTextRemainingTime,
				 iSliderPaneRect,
				 EAknLogicalFontPrimarySmallFont,//EAknLogicalFontSecondaryFont,
				 0,                         // C
				 88,                        // l
				 6,                         // r
				 30,                        // B
				 82,                        // W
				 ELayoutAlignRight );

    //ETextTotalTime
    AknLayoutUtils::LayoutLabel( iTextTotalTime,
				 iSliderPaneRect,
				 EAknLogicalFontSecondaryFont,
				 0,                         // C
				 44,                        // l
				 44,                        // r
				 31,                        // B
				 82,                        // W
				 ELayoutAlignCenter );

    // Scale icons

    if ( iSliderBackground )
    {
        AknIconUtils::SetSize(
            iSliderBackground->Bitmap(),
            iSliderBackgroundRect.Size(),
            EAspectRatioNotPreserved );

        UpdateProgressBarGraphics();

        if ( iAlive )
        {
            MTouchFeedback* feedback = MTouchFeedback::Instance();
            if ( feedback )
              feedback->SetFeedbackArea( this, 0, iSliderPaneRect,
                    ETouchFeedbackBasic, ETouchEventStylusDown);
        }
    }

    // Update label colors according to skin (ignore the color in layout data)
    MAknsSkinInstance* skin = AknsUtils::SkinInstance();
    TRgb color = KRgbBlack;

    // Color is not updated if it not found from the skin
    AknsUtils::GetCachedColor( skin, color, KAknsIIDQsnTextColors, EAknsCIQsnTextColorsCG6 );

    AknLayoutUtils::OverrideControlColorL( *iTextElapsedTime, EColorLabelText, color );

    AknLayoutUtils::OverrideControlColorL( *iTextRemainingTime, EColorLabelText, color );

    AknLayoutUtils::OverrideControlColorL( *iTextTotalTime, EColorLabelText, color );
}


TInt CMusicProgressBar::CallSetPosition()
{
   iParent->DoSeekEnd();

   iSeekTimer->Cancel();
   return 0;
}

TInt CMusicProgressBar::CallSetPosition( TAny* aAny )
{
   return ((CMusicProgressBar*)aAny)->CallSetPosition();
}


void CMusicProgressBar::HideTimeIndicators(TBool aHide)
{
   iHideTimeIndicators = aHide;
}

//  End of File
