#include "demac.h"

#include "FolderPlayView.h"
#include "FolderPlayEngine.h"


bool CPlayEngine::ParseMAC()
{
    struct ape_ctx_t ape_ctx;
    TInt  bytesinbuffer;
    TUint currentframe;
    TUint samplestoskip;
    TUint blockstodecode;
    TUint samplesdone;
    TInt firstbyte;
    TInt nblocks;
    TInt bytesconsumed;
    TInt n, i;

    TInt32* dec0(NULL);
    TInt32* dec1(NULL);
    TUint blkstodec(0);

    TInt aPos;
    TInt16 sample16;

    TInt16* filterbuf64((TInt16*)(((APE_buffers *)iEngineBuffers.Ptr())->filterbuf64));
    TInt16* filterbuf256((TInt16*)(((APE_buffers *)iEngineBuffers.Ptr())->filterbuf256));
    TUint32* seektablebuf((TUint32*)(((APE_buffers *)iEngineBuffers.Ptr())->seektablebuf));
    TInt32* decoded0((TInt32*)(((APE_buffers *)iEngineBuffers.Ptr())->decoded0));
    TInt32* decoded1((TInt32*)(((APE_buffers *)iEngineBuffers.Ptr())->decoded1));
    TUint8* inbuffer((TUint8*)(((APE_buffers *)iEngineBuffers.Ptr())->inbuffer));

    /* Read the file headers to populate the ape_ctx struct */
    read_filebuf(inbuffer, APE_INPUT_CHUNKSIZE);
    if (ape_parseheaderbuf(inbuffer,&ape_ctx) < 0) return EFalse;

    if (ape_ctx.compressiontype == 5000) return EFalse;

    TInt aBpS = ape_ctx.bps;
    iDataType.Set(aBpS <= 8 ? KMMFFourCCCodePCMU8 : (aBpS <= 16 ? KMMFFourCCCodePCM16 : KMMFFourCCCodeNULL));

    iSampleRate = Freq2SR(ape_ctx.samplerate);

    iChannels = ape_ctx.channels == 2 ? TMdaAudioDataSettings::EChannelsStereo :
		(ape_ctx.channels == 1 ? TMdaAudioDataSettings::EChannelsMono : 0);

    if(iDataType == KMMFFourCCCodeNULL || iSampleRate == 0 || iChannels == 0) return EFalse;

    ScheduleAudioReset();

    iParent.SetTotalLength( ((TInt64)((ape_ctx.totalframes - 1) * ape_ctx.blocksperframe) + ape_ctx.finalframeblocks) * 1000 / ape_ctx.samplerate);

    /* Initialize the seektable for this file */
    ape_ctx.seektable = seektablebuf;
    ape_ctx.numseekpoints = Min(APE_MAX_SEEKPOINTS, ape_ctx.numseekpoints);

    iAudioFile.Seek(ESeekStart, ape_ctx.seektablefilepos);
    if(read_filebuf((TUint8*)ape_ctx.seektable, ape_ctx.numseekpoints * sizeof(uint32_t)) <= 0)
       return EFalse;

    currentframe = 0; samplesdone = 0; samplestoskip = 0;
    firstbyte = 3;  /* Take account of the little-endian 32-bit byte ordering */

    if( iParent.iCurrentPos != 0)
    {
	samplestoskip = iParent.RecalcPos( ape_ctx.samplerate );

        TUint n = samplestoskip / ape_ctx.blocksperframe;

        if (n < ape_ctx.numseekpoints)
        {
          currentframe = n;

          aPos = ape_ctx.seektable[n];

          samplestoskip = samplestoskip - (n * ape_ctx.blocksperframe);

          samplesdone = currentframe * ape_ctx.blocksperframe;

          /* APE's bytestream is weird... */
          firstbyte = 3 - (aPos & 3);
          aPos &= ~3;
        }

    }
    else aPos = ape_ctx.firstframe;

    iAudioFile.Seek(ESeekStart, aPos);
    bytesinbuffer = read_filebuf(inbuffer, APE_INPUT_CHUNKSIZE);
    if(bytesinbuffer <= 0) return EFalse;

    while (iParent.iAlive && (currentframe < ape_ctx.totalframes))
    {
        /* Calculate how many blocks there are in this frame */
        if (currentframe == (ape_ctx.totalframes - 1))
            nblocks = ape_ctx.finalframeblocks;
        else
            nblocks = ape_ctx.blocksperframe;

        ape_ctx.currentframeblocks = nblocks;

        /* Initialise the frame decoder */
//        init_frame_decoder(&ape_ctx, inbuffer, &firstbyte, &bytesconsumed);

        init_entropy_decoder(&ape_ctx, inbuffer, &firstbyte, &bytesconsumed);

        init_predictor_decoder(&ape_ctx.predictor);

        switch (ape_ctx.compressiontype)
        {
        case 2000:
            init_filter_16_11(filterbuf64);
            break;

        case 3000:
            init_filter_64_11(filterbuf64);
            break;

        case 4000:
            init_filter_256_13(filterbuf256);
            init_filter_32_10(filterbuf64);
            break;
        }

        /* Update buffer */
        memmove(inbuffer,inbuffer + bytesconsumed, bytesinbuffer - bytesconsumed);
        bytesinbuffer -= bytesconsumed;

        n = read_filebuf(inbuffer + bytesinbuffer, APE_INPUT_CHUNKSIZE - bytesinbuffer);
        bytesinbuffer += n;

        /* Decode the frame a chunk at a time */
        while (iParent.iAlive && nblocks > 0)
        {
            blockstodecode = Min(APE_BLOCKS_PER_LOOP, nblocks);

            if (decode_chunk(&ape_ctx, inbuffer, &firstbyte,
                                    &bytesconsumed,
                                    decoded0, decoded1,
                                    blockstodecode) < 0) return EFalse;

            if (samplestoskip > 0) {
                if (samplestoskip < blockstodecode) {
		    dec0 = decoded0 + samplestoskip;
                    dec1 = decoded1 + samplestoskip;
                    blkstodec = blockstodecode - samplestoskip;
                    samplestoskip = 0;
                } else {
                    samplestoskip -= blockstodecode;
                    goto _skip; //dsv
                }
            } else {
                dec0 = decoded0;
                dec1 = decoded1;
                blkstodec = blockstodecode;
            }

            TUint8* aIt = (TUint8*)iCurBuffer.Ptr();
            TUint8* aEnd = aIt + iCurBuffer.MaxLength();

            TInt aBps(ape_ctx.bps);
            TInt aChannels(ape_ctx.channels);
            TBool aStereo(!((aChannels==1) || ((ape_ctx.frameflags & (APE_FRAMECODE_PSEUDO_STEREO|APE_FRAMECODE_STEREO_SILENCE)) == APE_FRAMECODE_PSEUDO_STEREO)));
            TInt left, right;

    for (i = 0 ; i < blkstodec ; i++)
    {
      if (aBps == 16) {
        if (aStereo)
        {
          /* Decorrelate output depth */
          left = dec1[i] - (dec0[i] >> 1);
          right = left + dec0[i];

          *(aIt++) = left;
          *(aIt++) = (left >> 8);

          *(aIt++) = right;
          *(aIt++) = (right >> 8);
        }
        else
        {
          sample16 = dec0[i];
          *(aIt++) = sample16;
          *(aIt++) = (sample16 >> 8);

          if (aChannels == 2) {
            sample16 = dec1[i];
            *(aIt++) = sample16;
            *(aIt++) = (sample16 >> 8);
          }
        }
      } else if (aBps == 8) {
        /* 8 bit WAV uses unsigned samples */
        if (aStereo)
        {
          left = dec1[i] - (dec0[i] >> 1);
          right = left + dec0[i];

          *(aIt++) = (left + 0x80) & 0xff;
          *(aIt++) = (right + 0x80) & 0xff;
        }
        else
        {
          *(aIt++) = (dec0[i] + 0x80) & 0xff;

          if (aChannels == 2) {
            *(aIt++) = (dec1[i] + 0x80) & 0xff;
          }
        }
      }

            if(aIt >= aEnd)
            {
               OutputBuffer();
               aIt = (TUint8*)iCurBuffer.Ptr();
               aEnd = aIt + iCurBuffer.MaxLength();
            }
    }
            iCurBuffer.Set(aIt, (aEnd - aIt), (aEnd - aIt));

_skip: // dsv
            samplesdone += blockstodecode;

            if (!samplestoskip) {
		iParent.iCurrentPos = (TInt64)samplesdone * 1000 / ape_ctx.samplerate;
            }

            /* Update the buffer */
            memmove(inbuffer,inbuffer + bytesconsumed, bytesinbuffer - bytesconsumed);
            bytesinbuffer -= bytesconsumed;

            n = read_filebuf(inbuffer + bytesinbuffer, APE_INPUT_CHUNKSIZE - bytesinbuffer);
            bytesinbuffer += n;

            /* Decrement the block count */
            nblocks -= blockstodecode;
        }

        currentframe++;
    }

    return ETrue;
}
