#include "wavpack.h"
#include "FolderPlayView.h"
#include "FolderPlayEngine.h"

#include <stdio.h>

static TInt32 read_bytes(RFile* aFile, void *data, TInt32 bcount)
{
    TPtr8 aBuffer((TUint8*)data, bcount);
    if(aFile->Read(aBuffer, bcount) != KErrNone)
 	return -1;
    else
	return aBuffer.Length();
}

static TUint32 get_pos(RFile* aFile)
{
    TInt curpos(0);
    aFile->Seek(ESeekCurrent, curpos);

    return (TUint32)curpos;
}

static TInt set_pos_rel(RFile* aFile, TInt32 delta, TInt mode)
{
   TInt aOffset(delta);

   return aFile->Seek(mode == SEEK_SET ? ESeekStart :
	(mode == SEEK_END ? ESeekEnd : ESeekCurrent), aOffset) == KErrNone ? 0 : -1;
}

static TInt set_pos_abs(RFile* aFile, TUint32 pos)
{
    return set_pos_rel(aFile, pos, SEEK_SET);
}

static TUint32 get_length(RFile* aFile)
{
    TInt filesize(0);

    aFile->Size(filesize);

    return (TUint32)filesize;
}

static TInt can_seek(void *id)
{
    return 1;
}

static WavpackStreamReader freader = {
	(int32_t (*)(void *id, void *data, int32_t bcount))	read_bytes,
	(uint32_t (*)(void *id))				get_pos,
	(int (*)(void *id, uint32_t pos))			set_pos_abs,
	(int (*)(void *id, int32_t delta, int mode))		set_pos_rel,
	(uint32_t (*)(void *id))				get_length,
								can_seek
};


bool CPlayEngine::ParseWavPack()
{
    WavpackContext *wpc;
    int error;
    TInt bps, nchans;
    TUint32 SampleRate;
    TUint32 nsamples;
    TInt32* temp_buf((TInt32 *)iEngineBuffers.Ptr());

    /* Create a decoder instance */
    wpc = WavpackOpenFileInputEx(&freader, (void *)&iAudioFile, NULL, &error, OPEN_2CH_MAX, 0);

    if(!wpc) return EFalse;

    bps = WavpackGetBitsPerSample(wpc);
    // supported only 16bit files
    iDataType.Set(bps <= 8 ? KMMFFourCCCodeNULL : (bps <= 16 ? KMMFFourCCCodePCM16 : KMMFFourCCCodeNULL));

    SampleRate = WavpackGetSampleRate(wpc);
    iSampleRate = Freq2SR(SampleRate);

    nchans = WavpackGetReducedChannels(wpc);
    iChannels = nchans == 2 ? TMdaAudioDataSettings::EChannelsStereo :
		(nchans == 1 ? TMdaAudioDataSettings::EChannelsMono : 0);


    if(iDataType == KMMFFourCCCodeNULL || iSampleRate == 0 || iChannels == 0) return EFalse;

    ScheduleAudioReset();

    iParent.SetTotalLength((TInt64)WavpackGetNumSamples(wpc) * 1000 / SampleRate);

    if( iParent.iCurrentPos != 0)
    {
	TUint skip = iParent.RecalcPos( SampleRate );

        if (!WavpackSeekSample(wpc, skip))
	{
          WavpackCloseFile(wpc);
          return EFalse;
        }
    }


    /* The main decoder loop */
    while(iParent.iAlive)
    {
        nsamples = WavpackUnpackSamples(wpc, temp_buf, WV_BUFFER_SIZE / nchans);

        if(nsamples == 0) break;

	TInt16* aIt = (TInt16*)iCurBuffer.Ptr();
	TInt16* aEnd = (TInt16*)(iCurBuffer.Ptr() + iCurBuffer.MaxLength());

	for(TUint i = 0; i < nsamples * nchans; i += nchans)
	{
		*aIt = (TInt16)temp_buf[i]; ++aIt;

		if( nchans == 2) { *aIt = (TInt16)temp_buf[i+1]; ++aIt; }

		if(aIt >= aEnd)
		{
			OutputBuffer();
			aIt = (TInt16*)iCurBuffer.Ptr();
			aEnd = (TInt16*)(iCurBuffer.Ptr() + iCurBuffer.MaxLength());
		}
	}
	TInt aNewSize = (aEnd - aIt) * 2;
	iCurBuffer.Set((TUint8*)aIt, aNewSize, aNewSize);

        iParent.iCurrentPos = (TInt64)WavpackGetSampleIndex(wpc) * 1000 / SampleRate;
    }

    WavpackCloseFile(wpc);

    return ETrue;
}
